@@ -21,9 +21,9 @@ except ModuleNotFoundError as e:
 
 logging.basicConfig(level=logging.DEBUG)
 
-USERNAME = 'faekie@hotmail.com'
-PASSWORD = 'VinterStorm2021!'
-PRINTRESPONSE = True
+USERNAME = 'email@domain.com'
+PASSWORD = 'password!'
+PRINTRESPONSE = False
 INTERVAL = 20
 
 COMPONENTS = {
@@ -35,13 +35,12 @@ COMPONENTS = {
 }
 
 RESOURCES = [
-        "adblue_level",
-	"auxiliary_climatisation",
-	"battery_level",
-        "min_charge_level",
-	"charge_max_ampere",
-	"charger_action_status",
-        "charging",
+		"adblue_level",
+		"auxiliary_climatisation",
+		"battery_level",
+		"charge_max_ampere",
+		"charger_action_status",
+		"charging",
         "charge_rate",
         "charging_power",
 		"charging_cable_connected",
@@ -153,7 +152,6 @@ async def main():
                 print(f"\tVIN: {vehicle.vin}")
                 print(f"\tModel: {vehicle.model}")
                 print(f"\tManufactured: {vehicle.model_year}")
-                print(f"\tModel image: {vehicle.model_image}")
                 print(f"\tConnect service deactivated: {vehicle.deactivated}")
                 if vehicle.is_nickname_supported: print(f"\tNickname: {vehicle.nickname}")
                 print(f"\tObject attributes, and methods:")
@@ -183,17 +181,6 @@ async def main():
 
         print('')
         print(f"Sleeping for {INTERVAL} seconds")
-        #data = {
-        #    'performAction': {
-        #        'quickstop': {
-        #            'active': False
-        #        }
-        #    }
-        #}
-        #spin = 1234
-        data = {'type': 'stop'}
-        resp = await connection.setCharging("TMBJW7NP0L7050944", data=data)
-        print(resp)
         time.sleep(INTERVAL)
 
         print('')
